########################
#
# Toth isotherm model This script is made freely available and this
# notice should be included with the distribution of this script.
# This script may be freely distributed and modified.
# Jeff Kenvin, April 2013
#
########################
import numpy as np
import mic
import scipy.optimize as optim
#
########
#
## data from sample file
p_ads, q_ads, n_ads, warm_fs, cold_fs, mass, desc = mic.isotherm('abs')
#csa, hsd, dcf, mol_weight, analysis_gas = mic.adsorptive_data(0)
#
## the n_ads is the number of adsorption points
p_ads = p_ads[0 : n_ads]            # only adsorption points for relative pressure
q_ads = q_ads[0 : n_ads]            # only adsorption points for volume adsorbed
#
# function to convert to mbar
def fmmHg2mbar( p ):
    pmBar = p * 1013.25 / 760.0
    return pmBar
# function to convert to mmoles
def fV2mmol( v ):
    nmmol = v/22.414
    return nmmol
#
x = fmmHg2mbar(p_ads)
xMin = np.min(x)
xMax = np.max(x)
xStep = (xMax-xMin)/100.0
xF = np.arange(xMin,xMax,xStep)
# convert to milli moles
y = fV2mmol(q_ads) 
#
# function to minimize for calc of Toth params -> nm*K*x/(1.0+(K*x)**t)**(1.0/t)
#
def fMinToth( par, x, y ):
    nm = par[0]
    K = par[1]
    t = par[2]
    err = y - nm*K*x/(1.0+(K*x)**t)**(1.0/t)
    rmse = np.sqrt(np.sum(err**2))
    return rmse
#
# function to calculate iso from Toth params
#
def fToth(par,x):
    nm = par[0]
    K = par[1]
    t = par[2]
    n =  nm*K*x/(1.0+(K*x)**t)**(1.0/t)
    return n
#
# use Langmuir to estimate nm,K
#
def fFitLang(x , y):
    ix = np.ones(x.shape)
    yf = x/y
    A = np.vstack([x,ix]).T
    m,b = np.linalg.lstsq(A,yf.T)[0]
    nm = 1/m
    K = m/b
    return nm,K
#
# create typical LF linear transforms
#
def fLinT( x,y,nm,K,t):
    yLt = np.log(y**t/(nm**t-y**t))
    yLfitT = t*np.log(x) + t*np.log(K)
    return yLt,yLfitT
#
nmL,Kl = fFitLang(x,y)
#
# Use Langmuir params as initial guess for Toth
#
t = 1.0
par = (nmL , Kl, t)
#
p = optim.fmin(fMinToth, par, args=(x,y), maxfun=10000,disp=0)
# 
# Toth params
#
nmT = p[0]
Kt = p[1]
t = p[2]
#
# predict isotherm from Toth params
#
yT = fToth(p,x)
yLtoth,yLfitToth = fLinT(x,y,nmT,Kt,t)
#
# Estimate Henry's Constant from Toth 
#
Kh = p[0]*p[1]
#
#########
#
# send data back to the MicroActive UI and Report system
#
#
#
def fTextOut( x, oFormat):
    tOut = [(oFormat % xOut) for xOut in x]
    return tOut
#
# Create a summary report
# Estimate Henry's Constant from Toth 
#
Kh = nmT*Kt
#
mic.summary('Summary')
mic.summary.add( 'Toth parameters', ["K (mmol/g)/mbar = ",
                                     'a, 1/mbar = ', 
                                     'Qm, mmol/g = ',
                                     't (exponent)= '
                                     ],
                 [ "%10.5f" % Kh,
                   "%10.5f" % Kt,
                  "%10.5f" % nmT, 
                  "%10.5f" % t 
                 ] )
# create plots
########
#
# create color table - this is a convenient table - it may not suit
# your sense of aesthetics.
#
black = "#000000"
blue = "#0000FF"
green = "#008000"
red = "#FF0000"
#
navy = "#000080"
teal = "#008080"
lime = "#00FF00"
aqua = "#00FFFF"
purple = "#800080"
gray = "#808080"
#
########
#
mic.graph('Model','Pressure, mbar','Qads, mmoles/g')
mic.graph.add('Isotherm', x , y, color = red, graphtype = 'points', marker = 'o')
mic.graph.add('Toth', x, yT , color = blue, linestyle = '-', graphtype = 'curve')
#
mic.graph('Linear Transform', 'ln[p]','ln[n^t/(nm^t-n^t)]', xlinear = 1, ylinear = 1)
mic.graph.add('Data', np.log(x) ,
              yLtoth,color = red, graphtype = 'points', marker = 'o')
mic.graph.add('Toth', np.log(x),
              yLfitToth , color = blue , linestyle = '-', graphtype = 'curve')
#
# create tabular output
#
mic.table(title='Tabular Report')
mic.table.addtable('Tabular Report')
mic.table.addcolumn('P, mbar', fTextOut( x, '%6.4f'))
mic.table.addcolumn('Qads, mmol/g', fTextOut(y, '%8.3f'))
mic.table.addcolumn('Q(Toth), mmol/g', fTextOut(yT, '%8.3f'))
mic.table.addcolumn('Residual, mmol/g', fTextOut((y-yT), '%8.3f'))
mic.table.addcolumn('% Different', fTextOut((y-yT)/y*100.0, '%8.3f'))

